/**********************************************************************
 *
 * Filename:    interrupt.c
 * 
 * Description: eCos interrupt program.
 *
 * Notes:       
 *
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

/**********************************************************************
 *
 * Notes:       This file is specific to the LDS4000 board
 * 
 * Copyright (c) 2007 Corebell System, Inc. All rights reserved.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://corebell.co.kr
 *
 **********************************************************************/
 
#include <cyg/kernel/kapi.h>
#include "stdint.h"
#include "pxa270.h"
#include "lds4000.h"
#include "timer.h"
#include "led.h"


#define LED_TASK_STACK_SIZE         (4096)
#define LED_TASK_PRIORITY           (12)


/* Declare the task variables. */
int8_t ledTaskStack[LED_TASK_STACK_SIZE];
cyg_thread ledTaskObj;
cyg_handle_t ledTaskHdl;


/**********************************************************************
 *
 * Function:    blinkLedTask
 *
 * Description: This task handles toggling the LED when it is
 *              signaled from the timer interrupt handler.
 *
 * Notes:       
 *
 * Returns:     None.
 *
 **********************************************************************/ 
void blinkLedTask(cyg_addrword_t data)
{
    while (1)
    {
        /* Wait for the signal that it is time to toggle the LED. */
        cyg_semaphore_wait(&ledToggleSemaphore);

        /* Change the state of the green LED. */
        ledToggle();
    }
}


/**********************************************************************
 *
 * Function:    cyg_user_start
 *
 * Description: Main routine for eCos interrupt Blinking LED program.
 *              This function creates the LED toggle semaphore, the
 *              LED task, and the timer interrupt handler.
 * 
 * Notes:       This routine invokes the scheduler upon exit.
 *
 * Returns:     None.
 *
 **********************************************************************/
void cyg_user_start(void)
{
    /* Configure the green LED control pin. */
    ledInit();

    /* Create the semaphore for the task signaling. This semaphore
     * is initialized with a value of 0 so the toggling task must wait
     * for the first time period to elapse. */
    cyg_semaphore_init(&ledToggleSemaphore, 0);

    /* Create the LED task. */
    cyg_thread_create(LED_TASK_PRIORITY,
                      blinkLedTask,
                      (cyg_addrword_t)0,
                      "LED Task",
                      (void *)ledTaskStack,
                      LED_TASK_STACK_SIZE,
                      &ledTaskHdl,
                      &ledTaskObj);

    /* Notify the scheduler to start running the task. */
    cyg_thread_resume(ledTaskHdl);

    /* Initialize the interrupt for the timer. */
    cyg_interrupt_create(timerInterruptVector,
                         0,
                         0,
                         timerIsr,
                         timerDsr,
                         &timerInterruptHdl,
                         &timerInterruptObj);

    cyg_interrupt_attach(timerInterruptHdl);
    cyg_interrupt_acknowledge(timerInterruptVector);
    cyg_interrupt_unmask(timerInterruptVector);

    /* Initialize the timer registers. */
    timerInit();
}

