/**********************************************************************
 *
 * Filename:    mailbox.c
 * 
 * Description: eCos mailbox program.
 *
 * Notes:       
 *
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

#include <cyg/kernel/kapi.h>
#include <cyg/infra/diag.h>
#include "stdint.h"
#include "pxa270.h"
#include "lds4000.h"
#include "button.h"
#include "led.h"


#define TICKS_PER_SECOND	        (100)

#define PRODUCER_TASK_STACK_SIZE    (4096)
#define PRODUCER_TASK_PRIORITY	    (12)

#define CONSUMER_TASK_STACK_SIZE    (4096)
#define CONSUMER_TASK_PRIORITY	    (12)


/* Declare the task variables. */
int8_t producerTaskStack[PRODUCER_TASK_STACK_SIZE];
cyg_thread producerTaskObj;
cyg_handle_t producerTaskHdl;

int8_t consumerTaskStack[CONSUMER_TASK_STACK_SIZE];
cyg_thread consumerTaskObj;
cyg_handle_t consumerTaskHdl;

cyg_handle_t mailboxHdl;
cyg_mbox mailbox;


/**********************************************************************
 *
 * Function:    producerTask
 *
 * Description: This task monitors button SW0. Once pressed, the button
 *              is debounced and a message is sent to the waiting
 *              consumer task.
 *
 * Notes:       This function is specific to the Arcom board. 
 *
 * Returns:     None.
 *
 **********************************************************************/ 
void producerTask(cyg_addrword_t data)
{
    uint32_t buttonPressCount = 0;
    int buttonOn;

    while (1)
    {
        /* Delay for 10 milliseconds. */
        cyg_thread_delay(TICKS_PER_SECOND / 100);

        /* Check if the SW0 button has been pressed. */
        buttonOn = buttonDebounce();

        /* If button SW0 was pressed, send a message to the consumer task. */
        if (buttonOn)
        {
            buttonPressCount++;
            cyg_mbox_put(mailboxHdl, (void *)buttonPressCount);
        }
    }
}


/**********************************************************************
 *
 * Function:    consumerTask
 *
 * Description: This task waits for a message from the producer task.
 *              Once the message is received via the mailbox, it
 *              outputs a message and toggles the green LED.
 *
 * Notes:       
 *
 * Returns:     None.
 *
 **********************************************************************/ 
void consumerTask(cyg_addrword_t data)
{
    uint32_t rcvMsg;

    while (1)
    {
        /* Wait for a new message. */
        rcvMsg = (uint32_t)cyg_mbox_get(mailboxHdl);

        diag_printf("Keypad Button pressed %d times.\n", rcvMsg);

        ledToggle();
    }
}


/**********************************************************************
 *
 * Function:    cyg_user_start
 *
 * Description: Main routine for the eCos mailbox program. This
 *              function creates the semaphore and the producer and
 *              consumer tasks.
 * 
 * Notes:       This routine invokes the scheduler upon exit.
 *
 * Returns:     None.
 *
 **********************************************************************/
void cyg_user_start(void)
{
    /* Configure the green LED control pin. */
    ledInit();

    /* Create the mailbox for sending messages between tasks. */
    cyg_mbox_create(&mailboxHdl, &mailbox);

    /* Create the producer and consumer tasks. */
    cyg_thread_create(PRODUCER_TASK_PRIORITY,
                      producerTask,
                      (cyg_addrword_t)0,
                      "Producer Task",
                      (void *)producerTaskStack,
                      PRODUCER_TASK_STACK_SIZE,
                      &producerTaskHdl,
                      &producerTaskObj);

    cyg_thread_create(CONSUMER_TASK_PRIORITY,
                      consumerTask,
                      (cyg_addrword_t)0,
                      "Consumer Task",
                      (void *)consumerTaskStack,
                      CONSUMER_TASK_STACK_SIZE,
                      &consumerTaskHdl,
                      &consumerTaskObj);

    /* Notify the scheduler to start running the tasks. */
    cyg_thread_resume(producerTaskHdl);
    cyg_thread_resume(consumerTaskHdl);

    diag_printf("eCos mailbox example - press button SW0.\n");
}

